@@ -12,10 +12,10 @@
 # include <config.h>
 #endif
 
-#include "session/hbci/s_verify_hbci.h"
-#include "session/s_decode.h"
-#include "session/cryptparams.h"
-#include "parser/parser.h"
+#include "libaqfints/session/hbci/s_verify_hbci.h"
+#include "libaqfints/session/s_decode.h"
+#include "libaqfints/session/cryptparams.h"
+#include "libaqfints/parser/parser.h"
 
 #include <gwenhywfar/misc.h>
 #include <gwenhywfar/debug.h>
