@@ -13,12 +13,12 @@
 #endif
 
 
-#include "jobgetbalance_p.h"
-#include "aqhbci_l.h"
+#include "jobgetbalance_l.h"
+#include "aqhbci/aqhbci_l.h"
 #include "accountjob_l.h"
-#include "job_l.h"
+#include "aqhbci/joblayer/job_l.h"
 #include "aqhbci/joblayer/job_crypt.h"
-#include "user_l.h"
+#include "aqhbci/banking/user_l.h"
 
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/misc.h>
@@ -30,16 +30,32 @@
 
 
 
-GWEN_INHERIT(AH_JOB, AH_JOB_GETBALANCE);
+/* ------------------------------------------------------------------------------------------------
+ * forward declarations
+ * ------------------------------------------------------------------------------------------------
+ */
+
+static int _jobApi_ProcessBankAccount(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx);
+static int _jobApi_ProcessInvestmentAccount(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx);
+
+static AB_BALANCE *_readBalance(GWEN_DB_NODE *dbBalance);
+static int _readSecurities(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx, const char *docType, int noted, GWEN_BUFFER *buf);
+
+static AB_VALUE *_readAmountFromResponseDb(GWEN_DB_NODE *dbBalance);
+static GWEN_DATE *_readDateFromResponseDb(GWEN_DB_NODE *dbBalance);
 
 
 
+/* ------------------------------------------------------------------------------------------------
+ * implementations
+ * ------------------------------------------------------------------------------------------------
+ */
+
+
 
-/* --------------------------------------------------------------- FUNCTION */
 AH_JOB *AH_Job_GetBalance_new(AB_PROVIDER *pro, AB_USER *u, AB_ACCOUNT *account)
 {
   AH_JOB *j;
-  AH_JOB_GETBALANCE *aj;
   GWEN_DB_NODE *dbArgs;
   int useCreditCardJob=0;
   int useInvestmentJob=0;
@@ -74,16 +90,13 @@ AH_JOB *AH_Job_GetBalance_new(AB_PROVIDER *pro, AB_USER *u, AB_ACCOUNT *account)
   if (!j)
     return 0;
 
-  GWEN_NEW_OBJECT(AH_JOB_GETBALANCE, aj);
-  GWEN_INHERIT_SETDATA(AH_JOB, AH_JOB_GETBALANCE, j, aj, AH_Job_GetBalance_FreeData);
-
   AH_Job_SetSupportedCommand(j, AB_Transaction_CommandGetBalance);
 
   /* overwrite some virtual functions */
   if (useInvestmentJob)
-    AH_Job_SetProcessFn(j, AH_Job_GetBalanceInvestment_Process);
+    AH_Job_SetProcessFn(j, _jobApi_ProcessInvestmentAccount);
   else
-    AH_Job_SetProcessFn(j, AH_Job_GetBalance_Process);
+    AH_Job_SetProcessFn(j, _jobApi_ProcessBankAccount);
   AH_Job_SetGetLimitsFn(j, AH_Job_GetLimits_EmptyLimits);
   AH_Job_SetHandleCommandFn(j, AH_Job_HandleCommand_Accept);
   AH_Job_SetHandleResultsFn(j, AH_Job_HandleResults_Empty);
@@ -101,96 +114,280 @@ AH_JOB *AH_Job_GetBalance_new(AB_PROVIDER *pro, AB_USER *u, AB_ACCOUNT *account)
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
-void GWENHYWFAR_CB AH_Job_GetBalance_FreeData(void *bp, void *p)
+AB_BALANCE *_readBalance(GWEN_DB_NODE *dbBalance)
 {
-  AH_JOB_GETBALANCE *aj;
-
-  aj=(AH_JOB_GETBALANCE *)p;
-  GWEN_FREE_OBJECT(aj);
-}
-
-
-
-/* --------------------------------------------------------------- FUNCTION */
-AB_BALANCE *AH_Job_GetBalance__ReadBalance(GWEN_DB_NODE *dbT)
-{
-  AB_VALUE *v;
-  const char *p;
   AB_BALANCE *bal;
-  int isCredit=0;
+  AB_VALUE *value;
+  GWEN_DATE *dt;
 
   bal=AB_Balance_new();
 
-  /* get isCredit */
-  p=GWEN_DB_GetCharValue(dbT, "debitMark", 0, 0);
-  if (p) {
-    if (strcasecmp(p, "D")==0 ||
-        strcasecmp(p, "RC")==0) {
-      isCredit=0;
-    }
-    else if (strcasecmp(p, "C")==0 ||
-             strcasecmp(p, "RD")==0)
-      isCredit=1;
-    else {
-      DBG_ERROR(AQHBCI_LOGDOMAIN, "Bad debit mark \"%s\"", p);
-      AB_Balance_free(bal);
-      return NULL;
-    }
-  }
+  value=_readAmountFromResponseDb(dbBalance);
+  AB_Balance_SetValue(bal, value);
+  AB_Value_free(value);
 
-  /* read date */
-  p=GWEN_DB_GetCharValue(dbT, "date", 0, 0);
-  if (p) {
-    GWEN_DATE *dt;
-
-    dt=GWEN_Date_fromStringWithTemplate(p, "YYYYMMDD");
-    if (dt) {
-      AB_Balance_SetDate(bal, dt);
-      GWEN_Date_free(dt);
-    }
-    else {
-      DBG_ERROR(AQHBCI_LOGDOMAIN, "Bad date \"%s\"", p);
-      AB_Balance_free(bal);
-      return NULL;
-    }
-  }
-  else {
-    GWEN_DATE *dt;
-
-    DBG_WARN(AQHBCI_LOGDOMAIN, "No date, using current date");
-    dt=GWEN_Date_CurrentDate();
-    assert(dt);
-    AB_Balance_SetDate(bal, dt);
-    GWEN_Date_free(dt);
-  }
-
-  /* get value */
-  v=AB_Value_fromDb(dbT);
-  if (!v) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN, "Error parsing value from DB");
-    AB_Balance_free(bal);
-    return NULL;
-  }
-  else {
-    if (!isCredit)
-      AB_Value_Negate(v);
-
-    AB_Balance_SetValue(bal, v);
-    AB_Value_free(v);
-  }
+  dt=_readDateFromResponseDb(dbBalance);
+  AB_Balance_SetDate(bal, dt);
+  GWEN_Date_free(dt);
 
   return bal;
 }
 
 
 
-/* --------------------------------------------------------------- FUNCTION */
-int AH_Job_GetBalance__ReadSecurities(AH_JOB *j,
-                                      AB_IMEXPORTER_CONTEXT *ctx,
-                                      const char *docType,
-                                      int noted,
-                                      GWEN_BUFFER *buf)
+AB_VALUE *_readAmountFromResponseDb(GWEN_DB_NODE *dbBalance)
+{
+  AB_VALUE *value;
+  const char *p;
+
+  /* get value */
+  value=AB_Value_fromDb(dbBalance);
+  if (!value) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Error parsing value from DB");
+    return NULL;
+  }
+
+  /* get isCredit */
+  p=GWEN_DB_GetCharValue(dbBalance, "debitMark", 0, 0);
+  if (p) {
+    if (strcasecmp(p, "D")==0 || strcasecmp(p, "RC")==0) {
+      AB_Value_Negate(value);
+    }
+    else if (strcasecmp(p, "C")==0 || strcasecmp(p, "RD")==0) {
+        /* nothing to do */
+    }
+    else {
+      DBG_ERROR(AQHBCI_LOGDOMAIN, "Bad debit mark \"%s\"", p);
+      AB_Value_free(value);
+      return NULL;
+    }
+  }
+
+  return value;
+}
+
+
+
+GWEN_DATE *_readDateFromResponseDb(GWEN_DB_NODE *dbBalance)
+{
+  GWEN_DATE *dt=NULL;
+  const char *p;
+
+  /* read date */
+  p=GWEN_DB_GetCharValue(dbBalance, "date", 0, 0);
+  if (p) {
+    dt=GWEN_Date_fromStringWithTemplate(p, "YYYYMMDD");
+    if (dt==NULL) {
+      DBG_ERROR(AQHBCI_LOGDOMAIN, "Bad date \"%s\"", p);
+      return NULL;
+    }
+  }
+  else {
+    DBG_WARN(AQHBCI_LOGDOMAIN, "No date, using current date");
+    dt=GWEN_Date_CurrentDate();
+    assert(dt);
+  }
+
+  return dt;
+}
+
+
+
+int _jobApi_ProcessBankAccount(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
+{
+  GWEN_DB_NODE *dbResponses;
+  GWEN_DB_NODE *dbCurr;
+  int rv;
+
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Processing JobGetBalance");
+
+  dbResponses=AH_Job_GetResponses(j);
+  assert(dbResponses);
+
+  /* search for "Balance" */
+  dbCurr=GWEN_DB_GetFirstGroup(dbResponses);
+  while (dbCurr) {
+    GWEN_DB_NODE *dbBalance;
+
+    rv=AH_Job_CheckEncryption(j, dbCurr);
+    if (rv) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (encryption)");
+      AH_Job_SetStatus(j, AH_JobStatusError);
+      return rv;
+    }
+    rv=AH_Job_CheckSignature(j, dbCurr);
+    if (rv) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (signature)");
+      AH_Job_SetStatus(j, AH_JobStatusError);
+      return rv;
+    }
+
+    dbBalance=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "data/balance");
+    if (!dbBalance)
+      dbBalance=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "data/balancecreditcard");
+    if (dbBalance) {
+      GWEN_DB_NODE *dbT;
+      AB_ACCOUNT *a;
+      AB_IMEXPORTER_ACCOUNTINFO *ai;
+
+      DBG_NOTICE(AQHBCI_LOGDOMAIN, "Got a balance");
+      if (GWEN_Logger_GetLevel(0)>=GWEN_LoggerLevel_Debug)
+        GWEN_DB_Dump(dbBalance, 2);
+
+      a=AH_AccountJob_GetAccount(j);
+      assert(a);
+      ai=AB_Provider_GetOrAddAccountInfoForAccount(ctx, a);
+
+      /* read booked balance */
+      dbT=GWEN_DB_GetGroup(dbBalance, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "booked");
+      if (dbT) {
+        AB_BALANCE *bal;
+
+        bal=_readBalance(dbT);
+        if (bal) {
+          AB_Balance_SetType(bal, AB_Balance_TypeBooked);
+          AB_ImExporterAccountInfo_AddBalance(ai, bal);
+        }
+      }
+
+      /* read noted balance */
+      dbT=GWEN_DB_GetGroup(dbBalance, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "noted");
+      if (dbT) {
+        AB_BALANCE *bal;
+
+        bal=_readBalance(dbT);
+        if (bal) {
+          AB_Balance_SetType(bal, AB_Balance_TypeNoted);
+          AB_ImExporterAccountInfo_AddBalance(ai, bal);
+        }
+      }
+
+#if 0
+      /* read credit Line */
+      dbT=GWEN_DB_GetGroup(dbBalance, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "creditLine");
+      if (dbT) {
+        AB_VALUE *v;
+
+        v=AB_Value_fromDb(dbT);
+        if (!v) {
+          DBG_ERROR(AQHBCI_LOGDOMAIN, "Error parsing value from DB");
+        }
+        else {
+          AB_AccountStatus_SetBankLine(acst, v);
+        }
+        AB_Value_free(v);
+      }
+#endif
+
+      break; /* break loop, we found the balance */
+    } /* if "Balance" */
+
+    dbCurr=GWEN_DB_GetNextGroup(dbCurr);
+  }
+
+  return 0;
+}
+
+
+
+int _jobApi_ProcessInvestmentAccount(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
+{
+  GWEN_DB_NODE *dbResponses;
+  GWEN_DB_NODE *dbCurr;
+  GWEN_BUFFER *tbooked;
+  int rv;
+
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Processing JobGetBalance");
+
+  dbResponses=AH_Job_GetResponses(j);
+  assert(dbResponses);
+
+  tbooked=GWEN_Buffer_new(0, 8192, 0, 1);
+
+  /* search for "Balance" */
+  dbCurr=GWEN_DB_GetFirstGroup(dbResponses);
+  while (dbCurr) {
+    GWEN_DB_NODE *dbBalance;
+
+    //GWEN_DB_Dump(dbCurr, 8);
+
+    rv=AH_Job_CheckEncryption(j, dbCurr);
+    if (rv) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (encryption)");
+      AH_Job_SetStatus(j, AH_JobStatusError);
+      return rv;
+    }
+    rv=AH_Job_CheckSignature(j, dbCurr);
+    if (rv) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (signature)");
+      AH_Job_SetStatus(j, AH_JobStatusError);
+      return rv;
+    }
+
+    dbBalance=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "data/BalanceInvestment");
+
+    if (dbBalance) {
+      const void *p;
+      unsigned int bs;
+
+      DBG_NOTICE(AQHBCI_LOGDOMAIN, "Got a balance");
+      if (GWEN_Logger_GetLevel(0)>=GWEN_LoggerLevel_Debug)
+        GWEN_DB_Dump(dbBalance, 2);
+
+      p=GWEN_DB_GetBinValue(dbBalance, "booked", 0, 0, 0, &bs);
+      if (p && bs)
+        GWEN_Buffer_AppendBytes(tbooked, p, bs);
+
+      break; /* break loop, we found the balance */
+    } /* if(dbBalance) */
+
+    dbCurr=GWEN_DB_GetNextGroup(dbCurr);
+  } /* while(dbCurr) */
+
+  GWEN_Buffer_Rewind(tbooked);
+  /* now the buffers contain data to be parsed by DBIOs */
+
+  /* read received securities */
+  if (GWEN_Buffer_GetUsedBytes(tbooked)) {
+
+    if (_readSecurities(j, ctx, "mt535", 0, tbooked)) {
+      GWEN_Buffer_free(tbooked);
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Error parsing received securities");
+      AH_Job_SetStatus(j, AH_JobStatusError);
+      return -1;
+    }
+  }
+
+  if (GWEN_Logger_GetLevel(AQHBCI_LOGDOMAIN)>=GWEN_LoggerLevel_Debug) {
+    GWEN_DB_NODE *gn;
+    AB_SECURITY *stmp;
+
+    DBG_INFO(AQHBCI_LOGDOMAIN, "*** Dumping securities *********************");
+    stmp=AB_ImExporterContext_GetFirstSecurity(ctx);
+    while (stmp) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "*** --------------------------------------");
+      gn=GWEN_DB_Group_new("security");
+      AB_Security_toDb(stmp, gn);
+      GWEN_DB_Dump(gn, 2);
+      if (gn)
+        GWEN_DB_Group_free(gn);
+      stmp=AB_Security_List_Next(stmp);
+    }
+    AB_Security_free(stmp);
+
+    DBG_INFO(AQHBCI_LOGDOMAIN, "*** End dumping securities *****************");
+  }
+
+  GWEN_Buffer_free(tbooked);
+  return 0;
+}
+
+
+/* the following is ancient code. We should probably switch to using AqBanking's import functions like in
+ * JobGetBalance, but I have no way of testing changes (I don't have a credit card for which HBCI is available)
+ * and this code seems to work, so I don't want to touch it ATM.
+ */
+int _readSecurities(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx, const char *docType, int noted, GWEN_BUFFER *buf)
 {
   GWEN_DBIO *dbio;
   GWEN_SYNCIO *sio;
@@ -203,7 +400,6 @@ int AH_Job_GetBalance__ReadSecurities(AH_JOB *j,
   uint32_t progressId;
   uint64_t cnt=0;
 
-
   a=AH_AccountJob_GetAccount(j);
   assert(a);
   u=AH_Job_GetUser(j);
@@ -212,9 +408,7 @@ int AH_Job_GetBalance__ReadSecurities(AH_JOB *j,
   dbio=GWEN_DBIO_GetPlugin("swift");
   if (!dbio) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Plugin SWIFT is not found");
-    GWEN_Gui_ProgressLog(0,
-                         GWEN_LoggerLevel_Error,
-                         I18N("Plugin \"SWIFT\" not found."));
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("Plugin \"SWIFT\" not found."));
     return AB_ERROR_PLUGIN_MISSING;
   }
 
@@ -224,22 +418,15 @@ int AH_Job_GetBalance__ReadSecurities(AH_JOB *j,
 
   db=GWEN_DB_Group_new("transactions");
   dbParams=GWEN_DB_Group_new("params");
-  GWEN_DB_SetCharValue(dbParams, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                       "type", docType);
+  GWEN_DB_SetCharValue(dbParams, GWEN_DB_FLAGS_OVERWRITE_VARS, "type", docType);
   if (AH_User_GetFlags(u) & AH_USER_FLAGS_KEEP_MULTIPLE_BLANKS)
-    GWEN_DB_SetIntValue(dbParams, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                        "keepMultipleBlanks", 1);
+    GWEN_DB_SetIntValue(dbParams, GWEN_DB_FLAGS_OVERWRITE_VARS, "keepMultipleBlanks", 1);
   else
-    GWEN_DB_SetIntValue(dbParams, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                        "keepMultipleBlanks", 0);
+    GWEN_DB_SetIntValue(dbParams, GWEN_DB_FLAGS_OVERWRITE_VARS, "keepMultipleBlanks", 0);
 
-  rv=GWEN_DBIO_Import(dbio, sio,
-                      db, dbParams,
-                      GWEN_PATH_FLAGS_CREATE_GROUP);
+  rv=GWEN_DBIO_Import(dbio, sio, db, dbParams, GWEN_PATH_FLAGS_CREATE_GROUP);
   if (rv<0) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN,
-              "Error parsing SWIFT %s (%d)",
-              docType, rv);
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Error parsing SWIFT %s (%d)", docType, rv);
     GWEN_DB_Group_free(dbParams);
     GWEN_DB_Group_free(db);
     GWEN_SyncIo_free(sio);
@@ -329,226 +516,3 @@ int AH_Job_GetBalance__ReadSecurities(AH_JOB *j,
 }
 
 
-/* --------------------------------------------------------------- FUNCTION */
-int AH_Job_GetBalance_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
-{
-  AH_JOB_GETBALANCE *aj;
-  GWEN_DB_NODE *dbResponses;
-  GWEN_DB_NODE *dbCurr;
-  int rv;
-
-  DBG_INFO(AQHBCI_LOGDOMAIN, "Processing JobGetBalance");
-
-  assert(j);
-  aj=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_GETBALANCE, j);
-  assert(aj);
-
-  dbResponses=AH_Job_GetResponses(j);
-  assert(dbResponses);
-
-  /* search for "Balance" */
-  dbCurr=GWEN_DB_GetFirstGroup(dbResponses);
-  while (dbCurr) {
-    GWEN_DB_NODE *dbBalance;
-
-    rv=AH_Job_CheckEncryption(j, dbCurr);
-    if (rv) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (encryption)");
-      AH_Job_SetStatus(j, AH_JobStatusError);
-      return rv;
-    }
-    rv=AH_Job_CheckSignature(j, dbCurr);
-    if (rv) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (signature)");
-      AH_Job_SetStatus(j, AH_JobStatusError);
-      return rv;
-    }
-
-    dbBalance=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "data/balance");
-    if (!dbBalance)
-      dbBalance=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "data/balancecreditcard");
-    if (dbBalance) {
-      GWEN_DB_NODE *dbT;
-      AB_ACCOUNT *a;
-      AB_IMEXPORTER_ACCOUNTINFO *ai;
-
-      DBG_NOTICE(AQHBCI_LOGDOMAIN, "Got a balance");
-      if (GWEN_Logger_GetLevel(0)>=GWEN_LoggerLevel_Debug)
-        GWEN_DB_Dump(dbBalance, 2);
-
-      a=AH_AccountJob_GetAccount(j);
-      assert(a);
-      ai=AB_ImExporterContext_GetOrAddAccountInfo(ctx,
-                                                  AB_Account_GetUniqueId(a),
-                                                  AB_Account_GetIban(a),
-                                                  AB_Account_GetBankCode(a),
-                                                  AB_Account_GetAccountNumber(a),
-                                                  AB_Account_GetAccountType(a));
-      assert(ai);
-
-
-      /* read booked balance */
-      dbT=GWEN_DB_GetGroup(dbBalance, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "booked");
-      if (dbT) {
-        AB_BALANCE *bal;
-
-        bal=AH_Job_GetBalance__ReadBalance(dbT);
-        if (bal) {
-          AB_Balance_SetType(bal, AB_Balance_TypeBooked);
-          AB_ImExporterAccountInfo_AddBalance(ai, bal);
-        }
-      }
-
-      /* read noted balance */
-      dbT=GWEN_DB_GetGroup(dbBalance, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "noted");
-      if (dbT) {
-        AB_BALANCE *bal;
-
-        bal=AH_Job_GetBalance__ReadBalance(dbT);
-        if (bal) {
-          AB_Balance_SetType(bal, AB_Balance_TypeNoted);
-          AB_ImExporterAccountInfo_AddBalance(ai, bal);
-        }
-      }
-
-#if 0
-      /* read credit Line */
-      dbT=GWEN_DB_GetGroup(dbBalance, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "creditLine");
-      if (dbT) {
-        AB_VALUE *v;
-
-        v=AB_Value_fromDb(dbT);
-        if (!v) {
-          DBG_ERROR(AQHBCI_LOGDOMAIN, "Error parsing value from DB");
-        }
-        else {
-          AB_AccountStatus_SetBankLine(acst, v);
-        }
-        AB_Value_free(v);
-      }
-#endif
-
-      break; /* break loop, we found the balance */
-    } /* if "Balance" */
-
-    dbCurr=GWEN_DB_GetNextGroup(dbCurr);
-  }
-
-  return 0;
-}
-
-
-
-/* --------------------------------------------------------------- FUNCTION */
-int AH_Job_GetBalanceInvestment_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx)
-{
-  AH_JOB_GETBALANCE *aj;
-  GWEN_DB_NODE *dbResponses;
-  GWEN_DB_NODE *dbCurr;
-  GWEN_BUFFER *tbooked;
-  AB_ACCOUNT *a;
-  AB_IMEXPORTER_ACCOUNTINFO *ai;
-  int rv;
-
-  DBG_INFO(AQHBCI_LOGDOMAIN, "Processing JobGetBalance");
-
-  assert(j);
-  aj=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_GETBALANCE, j);
-  assert(aj);
-
-  dbResponses=AH_Job_GetResponses(j);
-  assert(dbResponses);
-
-  tbooked=GWEN_Buffer_new(0, 8192, 0, 1);
-
-  /* search for "Balance" */
-  dbCurr=GWEN_DB_GetFirstGroup(dbResponses);
-  while (dbCurr) {
-    GWEN_DB_NODE *dbBalance;
-
-    //GWEN_DB_Dump(dbCurr, 8);
-
-    rv=AH_Job_CheckEncryption(j, dbCurr);
-    if (rv) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (encryption)");
-      AH_Job_SetStatus(j, AH_JobStatusError);
-      return rv;
-    }
-    rv=AH_Job_CheckSignature(j, dbCurr);
-    if (rv) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (signature)");
-      AH_Job_SetStatus(j, AH_JobStatusError);
-      return rv;
-    }
-
-    dbBalance=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                               "data/BalanceInvestment");
-
-    if (dbBalance) {
-      const void *p;
-      unsigned int bs;
-
-      DBG_NOTICE(AQHBCI_LOGDOMAIN, "Got a balance");
-      if (GWEN_Logger_GetLevel(0)>=GWEN_LoggerLevel_Debug)
-        GWEN_DB_Dump(dbBalance, 2);
-
-      p=GWEN_DB_GetBinValue(dbBalance, "booked", 0, 0, 0, &bs);
-      if (p && bs)
-        GWEN_Buffer_AppendBytes(tbooked, p, bs);
-
-      break; /* break loop, we found the balance */
-    } /* if(dbBalance) */
-
-    dbCurr=GWEN_DB_GetNextGroup(dbCurr);
-  } /* while(dbCurr) */
-
-  GWEN_Buffer_Rewind(tbooked);
-
-  /* now the buffers contain data to be parsed by DBIOs */
-  a=AH_AccountJob_GetAccount(j);
-  assert(a);
-  ai=AB_ImExporterContext_GetOrAddAccountInfo(ctx,
-                                              AB_Account_GetUniqueId(a),
-                                              AB_Account_GetIban(a),
-                                              AB_Account_GetBankCode(a),
-                                              AB_Account_GetAccountNumber(a),
-                                              AB_Account_GetAccountType(a));
-  assert(ai);
-
-  /* read received securities */
-  if (GWEN_Buffer_GetUsedBytes(tbooked)) {
-
-    if (AH_Job_GetBalance__ReadSecurities(j, ctx, "mt535", 0, tbooked)) {
-      GWEN_Buffer_free(tbooked);
-      DBG_INFO(AQHBCI_LOGDOMAIN, "Error parsing received securities");
-      AH_Job_SetStatus(j, AH_JobStatusError);
-      return -1;
-    }
-  }
-
-  if (GWEN_Logger_GetLevel(AQHBCI_LOGDOMAIN)>=GWEN_LoggerLevel_Debug) {
-    GWEN_DB_NODE *gn;
-    AB_SECURITY *stmp;
-
-    DBG_INFO(AQHBCI_LOGDOMAIN, "*** Dumping securities *********************");
-    stmp=AB_ImExporterContext_GetFirstSecurity(ctx);
-    while (stmp) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "*** --------------------------------------");
-      gn=GWEN_DB_Group_new("security");
-      AB_Security_toDb(stmp, gn);
-      GWEN_DB_Dump(gn, 2);
-      if (gn)
-        GWEN_DB_Group_free(gn);
-      stmp=AB_Security_List_Next(stmp);
-    }
-    AB_Security_free(stmp);
-
-    DBG_INFO(AQHBCI_LOGDOMAIN, "*** End dumping securities *****************");
-  }
-
-  GWEN_Buffer_free(tbooked);
-  return 0;
-}
-
-
-
