@@ -15,14 +15,15 @@
 
 
 #include "dlg_edituserrdh_p.h"
-#include "banking/provider_l.h"
-#include "i18n_l.h"
+
+#include "aqhbci/banking/provider_l.h"
 
 #include "aqhbci/banking/user.h"
 #include "aqhbci/banking/provider.h"
 #include "aqhbci/banking/provider_online.h"
 #include "aqhbci/banking/provider_iniletter.h"
 
+#include "aqbanking/i18n_l.h"
 #include <aqbanking/backendsupport/user.h>
 #include <aqbanking/banking_be.h>
 #include <aqbanking/dialogs/dlg_selectbankinfo.h>
