@@ -11,7 +11,7 @@
 #ifndef AH_JOBQUEUE_L_H
 #define AH_JOBQUEUE_L_H
 
-#include "job_l.h"
+#include "aqhbci/joblayer/job_l.h"
 
 
 typedef struct AH_JOBQUEUE AH_JOBQUEUE;
@@ -53,10 +53,12 @@ typedef struct AH_JOBQUEUE AH_JOBQUEUE;
 /** bad pin flagged by server */
 #define AH_JOBQUEUE_FLAGS_BAD_PIN        0x01000000
 
+#define AH_JOBQUEUE_FLAGS_IGNOREACCOUNTS 0x02000000
+
 
 #include <gwenhywfar/misc.h>
 
-#include "message_l.h"
+#include "aqhbci/msglayer/message_l.h"
 
 typedef enum {
   AH_JobQueueAddResultOk=0,
@@ -85,6 +87,7 @@ AH_JOB *AH_JobQueue_GetFirstJob(const AH_JOBQUEUE *jq);
 unsigned int AH_JobQueue_GetCount(const AH_JOBQUEUE *jq);
 
 void AH_JobQueue_Dump(AH_JOBQUEUE *jq, FILE *f, unsigned int insert);
+void AH_JobQueue_DumpJobList(const AH_JOBQUEUE *jq, FILE *f, unsigned int insert);
 
 
 
@@ -114,6 +117,13 @@ GWEN_STRINGLIST *AH_JobQueue_GetSigners(const AH_JOBQUEUE *jq);
 void AH_JobQueue_SetSigners(AH_JOBQUEUE *jq, GWEN_STRINGLIST *signers);
 
 
+AH_JOBQUEUE *AH_JobQueue_GetReferenceQueue(const AH_JOBQUEUE *jq);
+/**
+ * Internally calls @ref AH_JobQueue_Attach.
+ */
+void AH_JobQueue_SetReferenceQueue(AH_JOBQUEUE *jq, AH_JOBQUEUE *refq);
+
+
 #endif /* AH_JOBQUEUE_L_H */
 
 
