@@ -25,6 +25,39 @@
 GWEN_INHERIT(AB_IMEXPORTER, AH_IMEXPORTER_CSV);
 
 
+static int _importCsv(AB_IMEXPORTER *ie,
+                      AB_IMEXPORTER_CONTEXT *ctx,
+                      GWEN_SYNCIO *sio,
+                      GWEN_DB_NODE *params);
+
+static int _exportCsv(AB_IMEXPORTER *ie,
+                      AB_IMEXPORTER_CONTEXT *ctx,
+                      GWEN_SYNCIO *sio,
+                      GWEN_DB_NODE *params);
+
+static int _checkCsv(AB_IMEXPORTER *ie, const char *fname);
+
+static int _getEditProfileDialog(AB_IMEXPORTER *ie,
+                                 GWEN_DB_NODE *params,
+                                 const char *testFileName,
+                                 GWEN_DIALOG **pDlg);
+
+static AB_VALUE *_valueFromDb(GWEN_DB_NODE *dbV, int commaThousands, int commaDecimal);
+static void _unsplitInOutValue(GWEN_DB_NODE *dbT, int commaThousands, int commaDecimal);
+static void _collectPurposeStrings(AB_TRANSACTION *t, GWEN_DB_NODE *dbT);
+static void _readValues(AB_TRANSACTION *t, GWEN_DB_NODE *dbT, int commaThousands, int commaDecimal);
+static void _readDates(AB_TRANSACTION *t, GWEN_DB_NODE *dbT, const char *dateFormat);
+static void _translateValuesSign(AB_TRANSACTION *t, GWEN_DB_NODE *dbT, GWEN_DB_NODE *dbParams);
+static int _mustNegate(GWEN_DB_NODE *dbT, GWEN_DB_NODE *dbParams);
+static void _switchLocalRemoteAccordingToSign(AB_TRANSACTION *t, int switchOnNegative);
+static int _groupNameMatches(const char *groupName, GWEN_DB_NODE *dbParams);
+
+static void GWENHYWFAR_CB _freeData(void *bp, void *p);
+
+static int _importFromGroup(AB_IMEXPORTER_CONTEXT *ctx, GWEN_DB_NODE *db, GWEN_DB_NODE *dbParams);
+
+
+
 
 AB_IMEXPORTER *AB_ImExporterCSV_new(AB_BANKING *ab)
 {
@@ -33,7 +66,7 @@ AB_IMEXPORTER *AB_ImExporterCSV_new(AB_BANKING *ab)
 
   ie=AB_ImExporter_new(ab, "csv");
   GWEN_NEW_OBJECT(AH_IMEXPORTER_CSV, ieh);
-  GWEN_INHERIT_SETDATA(AB_IMEXPORTER, AH_IMEXPORTER_CSV, ie, ieh, AH_ImExporterCSV_FreeData);
+  GWEN_INHERIT_SETDATA(AB_IMEXPORTER, AH_IMEXPORTER_CSV, ie, ieh, _freeData);
   ieh->dbio=GWEN_DBIO_GetPlugin("csv");
   if (!ieh->dbio) {
     DBG_ERROR(AQBANKING_LOGDOMAIN, "GWEN DBIO plugin \"CSV\" not available");
@@ -41,10 +74,10 @@ AB_IMEXPORTER *AB_ImExporterCSV_new(AB_BANKING *ab)
     return NULL;
   }
 
-  AB_ImExporter_SetImportFn(ie, AH_ImExporterCSV_Import);
-  AB_ImExporter_SetExportFn(ie, AH_ImExporterCSV_Export);
-  AB_ImExporter_SetCheckFileFn(ie, AH_ImExporterCSV_CheckFile);
-  AB_ImExporter_SetGetEditProfileDialogFn(ie, AH_ImExporterCSV_GetEditProfileDialog);
+  AB_ImExporter_SetImportFn(ie, _importCsv);
+  AB_ImExporter_SetExportFn(ie, _exportCsv);
+  AB_ImExporter_SetCheckFileFn(ie, _checkCsv);
+  AB_ImExporter_SetGetEditProfileDialogFn(ie, _getEditProfileDialog);
 
   /* announce special features */
   AB_ImExporter_AddFlags(ie, AB_IMEXPORTER_FLAGS_GETPROFILEEDITOR_SUPPORTED);
@@ -54,7 +87,7 @@ AB_IMEXPORTER *AB_ImExporterCSV_new(AB_BANKING *ab)
 
 
 
-void GWENHYWFAR_CB AH_ImExporterCSV_FreeData(void *bp, void *p)
+void GWENHYWFAR_CB _freeData(void *bp, void *p)
 {
   AH_IMEXPORTER_CSV *ieh;
 
@@ -65,10 +98,10 @@ void GWENHYWFAR_CB AH_ImExporterCSV_FreeData(void *bp, void *p)
 
 
 
-int AH_ImExporterCSV_Import(AB_IMEXPORTER *ie,
-                            AB_IMEXPORTER_CONTEXT *ctx,
-                            GWEN_SYNCIO *sio,
-                            GWEN_DB_NODE *params)
+int _importCsv(AB_IMEXPORTER *ie,
+               AB_IMEXPORTER_CONTEXT *ctx,
+               GWEN_SYNCIO *sio,
+               GWEN_DB_NODE *params)
 {
   AH_IMEXPORTER_CSV *ieh;
   GWEN_DB_NODE *dbData;
@@ -109,7 +142,7 @@ int AH_ImExporterCSV_Import(AB_IMEXPORTER *ie,
   }
   GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Notice,
                        "Transforming data to transactions");
-  rv=AH_ImExporterCSV__ImportFromGroup(ctx, dbData, params);
+  rv=_importFromGroup(ctx, dbData, params);
   if (rv) {
     GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
                          "Error importing data");
@@ -123,7 +156,280 @@ int AH_ImExporterCSV_Import(AB_IMEXPORTER *ie,
 
 
 
-AB_VALUE *AH_ImExporterCSV__ValueFromDb(GWEN_DB_NODE *dbV, int commaThousands, int commaDecimal)
+int _importFromGroup(AB_IMEXPORTER_CONTEXT *ctx, GWEN_DB_NODE *db, GWEN_DB_NODE *dbParams)
+{
+  GWEN_DB_NODE *dbT;
+  AB_TRANSACTION_TYPE defaultType=AB_Transaction_TypeStatement;
+  const char *dateFormat;
+  int usePosNegField;
+  int splitValueInOut;
+  int switchLocalRemote;
+  int switchOnNegative;
+  int commaThousands=0;
+  int commaDecimal=0;
+  const char *s;
+
+  dateFormat=GWEN_DB_GetCharValue(dbParams, "dateFormat", 0, "YYYY/MM/DD");
+  usePosNegField=GWEN_DB_GetIntValue(dbParams, "usePosNegField", 0, 0);
+  splitValueInOut=GWEN_DB_GetIntValue(dbParams, "splitValueInOut", 0, 0);
+  switchLocalRemote=GWEN_DB_GetIntValue(dbParams, "switchLocalRemote", 0, 0);
+  switchOnNegative=GWEN_DB_GetIntValue(dbParams, "switchOnNegative", 0, 1);
+
+  s=GWEN_DB_GetCharValue(dbParams, "commaThousands", 0, 0);
+  if (s)
+    commaThousands=*s;
+  s=GWEN_DB_GetCharValue(dbParams, "commaDecimal", 0, 0);
+  if (s)
+    commaDecimal=*s;
+
+  s=GWEN_DB_GetCharValue(dbParams, "transactionType", 0, "statement");
+  if (s && *s) {
+    defaultType=AB_Transaction_Type_fromString(s);
+    if (defaultType==AB_Transaction_TypeUnknown) {
+      DBG_ERROR(AQBANKING_LOGDOMAIN, "Invalid default transaction type \"%s\", assuming \"statement\"", s);
+      defaultType=AB_Transaction_TypeStatement;
+    }
+  }
+
+  dbT=GWEN_DB_GetFirstGroup(db);
+  while (dbT) {
+    if (_groupNameMatches(GWEN_DB_GroupName(dbT), dbParams)) {
+      /* possibly merge in/out values */
+      if (splitValueInOut)
+	_unsplitInOutValue(dbT, commaThousands, commaDecimal);
+
+      if (GWEN_DB_GetCharValue(dbT, "value/value", 0, NULL) || GWEN_DB_GetCharValue(dbT, "units", 0, NULL)) {
+        AB_TRANSACTION *t;
+
+        t=AB_Transaction_fromDb(dbT);
+        if (!t) {
+          DBG_ERROR(AQBANKING_LOGDOMAIN, "Error in imported transaction data");
+          GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, "Error in imported transaction data");
+          return GWEN_ERROR_GENERIC;
+        }
+
+	_collectPurposeStrings(t, dbT);
+	_readValues(t, dbT, commaThousands, commaDecimal);
+	_readDates(t, dbT, dateFormat);
+
+	if (usePosNegField)
+	  _translateValuesSign(t, dbT, dbParams);
+
+	if (switchLocalRemote)
+	  _switchLocalRemoteAccordingToSign(t, switchOnNegative);
+
+	if (AB_Transaction_GetType(t)<=AB_Transaction_TypeNone)
+	  AB_Transaction_SetType(t, defaultType);
+
+        DBG_DEBUG(AQBANKING_LOGDOMAIN, "Adding transaction");
+        AB_ImExporterContext_AddTransaction(ctx, t);
+      }
+      else {
+        DBG_INFO(AQBANKING_LOGDOMAIN, "Empty group (i.e. empty line in imported file)");
+      }
+    }
+    else {
+      int rv;
+
+      DBG_INFO(AQBANKING_LOGDOMAIN, "Not a transaction, checking subgroups");
+      /* not a transaction, check subgroups */
+      rv=_importFromGroup(ctx, dbT, dbParams);
+      if (rv) {
+        DBG_INFO(AQBANKING_LOGDOMAIN, "here");
+        return rv;
+      }
+    }
+
+    dbT=GWEN_DB_GetNextGroup(dbT);
+  } // while
+
+  return 0;
+}
+
+
+
+void _unsplitInOutValue(GWEN_DB_NODE *dbT, int commaThousands, int commaDecimal)
+{
+  AB_VALUE *tv=NULL;
+  const char *s;
+  const char *tc;
+
+  tc=GWEN_DB_GetCharValue(dbT, "value/currency", 0, NULL);
+  s=GWEN_DB_GetCharValue(dbT, "valueIn/value", 0, 0);
+  if (s && *s) {
+    GWEN_DB_NODE *dbV;
+
+    dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "valueIn");
+    tv=_valueFromDb(dbV, commaThousands, commaDecimal);
+  }
+  else {
+    s=GWEN_DB_GetCharValue(dbT, "valueOut/value", 0, 0);
+    if (s && *s) {
+      GWEN_DB_NODE *dbV;
+
+      dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "valueOut");
+      if (dbV) {
+        tv=_valueFromDb(dbV, commaThousands, commaDecimal);
+        if (!AB_Value_IsNegative(tv))
+          /* outgoing but positive, negate */
+          AB_Value_Negate(tv);
+      }
+    }
+  }
+
+  if (tv) {
+    GWEN_DB_NODE *dbTV;
+
+    if (tc)
+      AB_Value_SetCurrency(tv, tc);
+    dbTV=GWEN_DB_GetGroup(dbT, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "value");
+    AB_Value_toDb(tv, dbTV);
+    AB_Value_free(tv);
+  }
+}
+
+
+
+void _collectPurposeStrings(AB_TRANSACTION *t, GWEN_DB_NODE *dbT)
+{
+  const char *p;
+
+  /* possibly translate purpose */
+  p=GWEN_DB_GetCharValue(dbT, "purpose", 1, 0); /* "1" is correct here! */
+  if (p) {
+    int i;
+  
+    /* there are multiple purpose lines, read them properly */
+    AB_Transaction_SetPurpose(t, NULL);
+    for (i=0; i<99; i++) {
+      p=GWEN_DB_GetCharValue(dbT, "purpose", i, NULL);
+      if (p && *p)
+	AB_Transaction_AddPurposeLine(t, p);
+    }
+  }
+}
+
+
+
+void _readValues(AB_TRANSACTION *t, GWEN_DB_NODE *dbT, int commaThousands, int commaDecimal)
+{
+  GWEN_DB_NODE *dbV;
+  AB_VALUE *v=NULL;
+
+  /* translate values */
+  dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "value");
+  if (dbV) {
+    v=_valueFromDb(dbV, commaThousands, commaDecimal);
+    AB_Transaction_SetValue(t, v);
+    AB_Value_free(v);
+  }
+
+  dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "unitPriceValue");
+  if (dbV) {
+    v=_valueFromDb(dbV, commaThousands, commaDecimal);
+    AB_Transaction_SetUnitPriceValue(t, v);
+    AB_Value_free(v);
+  }
+
+  dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "fees");
+  if (dbV) {
+    v=_valueFromDb(dbV, commaThousands, commaDecimal);
+    AB_Transaction_SetFees(t, v);
+    AB_Value_free(v);
+  }
+
+  dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "taxes");
+  if (dbV) {
+    v=_valueFromDb(dbV, commaThousands, commaDecimal);
+    AB_Transaction_SetTaxes(t, v);
+    AB_Value_free(v);
+  }
+
+  dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "commissionValue");
+  if (dbV) {
+    v=_valueFromDb(dbV, commaThousands, commaDecimal);
+    AB_Transaction_SetCommissionValue(t, v);
+    AB_Value_free(v);
+  }
+}
+
+
+
+void _readDates(AB_TRANSACTION *t, GWEN_DB_NODE *dbT, const char *dateFormat)
+{
+  const char *p;
+
+  p=GWEN_DB_GetCharValue(dbT, "date", 0, 0);
+  if (p) {
+    GWEN_DATE *da;
+  
+    da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
+    if (da)
+      AB_Transaction_SetDate(t, da);
+    GWEN_Date_free(da);
+  }
+  
+  p=GWEN_DB_GetCharValue(dbT, "valutaDate", 0, 0);
+  if (p) {
+    GWEN_DATE *da;
+  
+    da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
+    if (da)
+      AB_Transaction_SetValutaDate(t, da);
+    GWEN_Date_free(da);
+  }
+  
+  p=GWEN_DB_GetCharValue(dbT, "mandateDate", 0, 0);
+  if (p) {
+    GWEN_DATE *dt;
+  
+    dt=GWEN_Date_fromStringWithTemplate(p, dateFormat);
+    if (dt) {
+      AB_Transaction_SetMandateDate(t, dt);
+      GWEN_Date_free(dt);
+    }
+  }
+  p=GWEN_DB_GetCharValue(dbT, "firstDate", 0, 0);
+  if (p) {
+    GWEN_DATE *da;
+  
+    da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
+    if (da)
+      AB_Transaction_SetFirstDate(t, da);
+    GWEN_Date_free(da);
+  }
+  p=GWEN_DB_GetCharValue(dbT, "lastDate", 0, 0);
+  if (p) {
+    GWEN_DATE *da;
+  
+    da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
+    if (da)
+      AB_Transaction_SetLastDate(t, da);
+    GWEN_Date_free(da);
+  }
+  p=GWEN_DB_GetCharValue(dbT, "nextDate", 0, 0);
+  if (p) {
+    GWEN_DATE *da;
+  
+    da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
+    if (da)
+      AB_Transaction_SetNextDate(t, da);
+    GWEN_Date_free(da);
+  }
+  p=GWEN_DB_GetCharValue(dbT, "unitPriceDate", 0, NULL);
+  if (p) {
+    GWEN_DATE *da;
+  
+    da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
+    if (da)
+      AB_Transaction_SetUnitPriceDate(t, da);
+    GWEN_Date_free(da);
+  }
+}
+
+
+
+AB_VALUE *_valueFromDb(GWEN_DB_NODE *dbV, int commaThousands, int commaDecimal)
 {
   const char *sv;
   const char *sc;
@@ -169,339 +475,150 @@ AB_VALUE *AH_ImExporterCSV__ValueFromDb(GWEN_DB_NODE *dbV, int commaThousands, i
 
 
 
-int AH_ImExporterCSV__ImportFromGroup(AB_IMEXPORTER_CONTEXT *ctx,
-                                      GWEN_DB_NODE *db,
-                                      GWEN_DB_NODE *dbParams)
+void _translateValuesSign(AB_TRANSACTION *t, GWEN_DB_NODE *dbT, GWEN_DB_NODE *dbParams)
+{
+  if (_mustNegate(dbT, dbParams)) {
+    const AB_VALUE *pv;
+
+    pv=AB_Transaction_GetValue(t);
+    if (pv) {
+      AB_VALUE *v;
+
+      v=AB_Value_dup(pv);
+      AB_Value_Negate(v);
+      AB_Transaction_SetValue(t, v);
+      AB_Value_free(v);
+    }
+
+    pv=AB_Transaction_GetUnits(t);
+    if (pv) {
+      AB_VALUE *v;
+
+      v=AB_Value_dup(pv);
+      AB_Value_Negate(v);
+      AB_Transaction_SetUnits(t, v);
+      AB_Value_free(v);
+    }
+  }
+}
+
+
+
+int _mustNegate(GWEN_DB_NODE *dbT, GWEN_DB_NODE *dbParams)
 {
-  GWEN_DB_NODE *dbT;
-  const char *dateFormat;
-  int usePosNegField;
-  int splitValueInOut;
-  int defaultIsPositive;
-  int switchLocalRemote;
-  int switchOnNegative;
   const char *posNegFieldName;
-  int commaThousands=0;
-  int commaDecimal=0;
-  uint32_t progressId;
+  int defaultIsPositive;
   const char *s;
 
-  dateFormat=GWEN_DB_GetCharValue(dbParams, "dateFormat", 0, "YYYY/MM/DD");
-  usePosNegField=GWEN_DB_GetIntValue(dbParams, "usePosNegField", 0, 0);
   defaultIsPositive=GWEN_DB_GetIntValue(dbParams, "defaultIsPositive", 0, 1);
   posNegFieldName=GWEN_DB_GetCharValue(dbParams, "posNegFieldName", 0, "posNeg");
-  splitValueInOut=GWEN_DB_GetIntValue(dbParams, "splitValueInOut", 0, 0);
+  
+  /* get positive/negative mark */
+  s=GWEN_DB_GetCharValue(dbT, posNegFieldName, 0, 0);
+  if (s) {
+    int j;
 
-  switchLocalRemote=GWEN_DB_GetIntValue(dbParams, "switchLocalRemote", 0, 0);
-  switchOnNegative=GWEN_DB_GetIntValue(dbParams, "switchOnNegative", 0, 1);
+    /* try positive marks first */
+    for (j=0; ; j++) {
+      const char *patt;
 
-  s=GWEN_DB_GetCharValue(dbParams, "commaThousands", 0, 0);
-  if (s)
-    commaThousands=*s;
-  s=GWEN_DB_GetCharValue(dbParams, "commaDecimal", 0, 0);
-  if (s)
-    commaDecimal=*s;
-
-  progressId=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
-                                    GWEN_GUI_PROGRESS_ALLOW_EMBED |
-                                    GWEN_GUI_PROGRESS_SHOW_PROGRESS |
-                                    GWEN_GUI_PROGRESS_SHOW_ABORT,
-                                    I18N("Importing parsed data..."),
-                                    NULL,
-                                    GWEN_DB_Groups_Count(db),
-                                    0);
-  dbT=GWEN_DB_GetFirstGroup(db);
-  while (dbT) {
-    int matches;
-    int i;
-    const char *p;
-    const char *gn;
-
-    /* check whether the name of the current groups matches */
-    matches=0;
-    gn=GWEN_DB_GroupName(dbT);
-    for (i=0; ; i++) {
-      p=GWEN_DB_GetCharValue(dbParams, "groupNames", i, 0);
-      if (!p)
-        break;
-      if (strcasecmp(gn, p)==0) {
-        matches=1;
-        break;
+      patt=GWEN_DB_GetCharValue(dbParams, "positiveValues", j, 0);
+      if (!patt)
+	break;
+      if (-1!=GWEN_Text_ComparePattern(s, patt, 0)) {
+          /* value already is positive, keep it that way */
+	return 0;
       }
     } /* for */
 
-    if (!matches && i==0) {
-      /* no names given, check default */
-      if ((strcasecmp(GWEN_DB_GroupName(dbT), "transaction")==0) ||
-          (strcasecmp(GWEN_DB_GroupName(dbT), "debitnote")==0) ||
-          (strcasecmp(GWEN_DB_GroupName(dbT), "line")==0))
-        matches=1;
-    }
+    for (j=0; ; j++) {
+      const char *patt;
 
-    if (matches) {
-      /* possibly merge in/out values */
-      if (splitValueInOut) {
-        AB_VALUE *tv=NULL;
-        const char *s;
-        const char *tc;
+      patt=GWEN_DB_GetCharValue(dbParams, "negativeValues", j, 0);
+      if (!patt)
+	break;
+      if (-1!=GWEN_Text_ComparePattern(s, patt, 0))
+	return 1;
+    } /* for */
+  }
 
-        tc=GWEN_DB_GetCharValue(dbT, "value/currency", 0, NULL);
-        s=GWEN_DB_GetCharValue(dbT, "valueIn/value", 0, 0);
-        if (s && *s) {
-          GWEN_DB_NODE *dbV;
-
-          dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "valueIn");
-          tv=AH_ImExporterCSV__ValueFromDb(dbV, commaThousands, commaDecimal);
-        }
-        else {
-          s=GWEN_DB_GetCharValue(dbT, "valueOut/value", 0, 0);
-          if (s && *s) {
-            GWEN_DB_NODE *dbV;
-
-            dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "valueOut");
-            if (dbV) {
-              tv=AH_ImExporterCSV__ValueFromDb(dbV, commaThousands, commaDecimal);
-              if (!AB_Value_IsNegative(tv))
-                /* outgoing but positive, negate */
-                AB_Value_Negate(tv);
-            }
-          }
-        }
-
-        if (tv) {
-          GWEN_DB_NODE *dbTV;
-
-          if (tc)
-            AB_Value_SetCurrency(tv, tc);
-          dbTV=GWEN_DB_GetGroup(dbT, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "value");
-          AB_Value_toDb(tv, dbTV);
-          AB_Value_free(tv);
-        }
-      }
-
-      if (GWEN_DB_GetCharValue(dbT, "value/value", 0, 0)) {
-        AB_TRANSACTION *t;
-        const char *p;
-        GWEN_DB_NODE *dbV;
-
-        DBG_DEBUG(AQBANKING_LOGDOMAIN, "Found a possible transaction");
-        t=AB_Transaction_fromDb(dbT);
-        if (!t) {
-          DBG_ERROR(AQBANKING_LOGDOMAIN, "Error in config file");
-          GWEN_Gui_ProgressLog(progressId, GWEN_LoggerLevel_Error,
-                               "Error in config file");
-          GWEN_Gui_ProgressEnd(progressId);
-          return GWEN_ERROR_GENERIC;
-        }
-
-        /* possibly translate purpose */
-        p=GWEN_DB_GetCharValue(dbT, "purpose", 1, 0); /* "1" is correct here! */
-        if (p) {
-          int i;
-
-          /* there are multiple purpose lines, read them properly */
-          AB_Transaction_SetPurpose(t, NULL);
-          for (i=0; i<99; i++) {
-            p=GWEN_DB_GetCharValue(dbT, "purpose", i, 0);
-            if (p && *p)
-              AB_Transaction_AddPurposeLine(t, p);
-          }
-        }
-
-        /* translate value */
-        dbV=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "value");
-        if (dbV) {
-          AB_VALUE *v;
-
-          v=AB_Value_fromDb(dbV);
-          AB_Transaction_SetValue(t, v);
-          AB_Value_free(v);
-        }
-
-        /* translate date */
-        p=GWEN_DB_GetCharValue(dbT, "date", 0, 0);
-        if (p) {
-          GWEN_DATE *da;
-
-          da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
-          if (da)
-            AB_Transaction_SetDate(t, da);
-          GWEN_Date_free(da);
-        }
-
-        /* translate valutaDate */
-        p=GWEN_DB_GetCharValue(dbT, "valutaDate", 0, 0);
-        if (p) {
-          GWEN_DATE *da;
-
-          da=GWEN_Date_fromStringWithTemplate(p, dateFormat);
-          if (da)
-            AB_Transaction_SetValutaDate(t, da);
-          GWEN_Date_free(da);
-        }
-
-        /* translate mandateDate */
-        p=GWEN_DB_GetCharValue(dbT, "mandateDate", 0, 0);
-        if (p) {
-          GWEN_DATE *dt;
-
-          dt=GWEN_Date_fromStringWithTemplate(p, dateFormat);
-          if (dt) {
-            AB_Transaction_SetMandateDate(t, dt);
-            GWEN_Date_free(dt);
-          }
-        }
-
-        /* possibly translate value */
-        if (usePosNegField) {
-          const char *s;
-          int determined=0;
-
-          /* get positive/negative mark */
-          s=GWEN_DB_GetCharValue(dbT, posNegFieldName, 0, 0);
-          if (s) {
-            int j;
-
-            /* try positive marks first */
-            for (j=0; ; j++) {
-              const char *patt;
-
-              patt=GWEN_DB_GetCharValue(dbParams, "positiveValues", j, 0);
-              if (!patt)
-                break;
-              if (-1!=GWEN_Text_ComparePattern(s, patt, 0)) {
-                /* value already is positive, keep it that way */
-                determined=1;
-                break;
-              }
-            } /* for */
-
-            if (!determined) {
-              for (j=0; ; j++) {
-                const char *patt;
-
-                patt=GWEN_DB_GetCharValue(dbParams, "negativeValues", j, 0);
-                if (!patt)
-                  break;
-                if (-1!=GWEN_Text_ComparePattern(s, patt, 0)) {
-                  const AB_VALUE *pv;
-
-                  /* value must be negated */
-                  pv=AB_Transaction_GetValue(t);
-                  if (pv) {
-                    AB_VALUE *v;
-
-                    v=AB_Value_dup(pv);
-                    AB_Value_Negate(v);
-                    AB_Transaction_SetValue(t, v);
-                    AB_Value_free(v);
-                  }
-                  determined=1;
-                  break;
-                }
-              } /* for */
-            }
-          }
-
-          /* still undecided? */
-          if (!determined && !defaultIsPositive) {
-            const AB_VALUE *pv;
-
-            /* value must be negated, because default is negative */
-            pv=AB_Transaction_GetValue(t);
-            if (pv) {
-              AB_VALUE *v;
-
-              v=AB_Value_dup(pv);
-              AB_Value_Negate(v);
-              AB_Transaction_SetValue(t, v);
-              AB_Value_free(v);
-            }
-          }
-
-        } /* if usePosNegField */
-
-        else if (switchLocalRemote) {
-          const AB_VALUE *pv;
-
-          /* value must be negated, because default is negative */
-          pv=AB_Transaction_GetValue(t);
-          if (pv) {
-            if (!(AB_Value_IsNegative(pv) ^ (switchOnNegative!=0))) {
-              const char *s;
-              GWEN_BUFFER *b1;
-              GWEN_BUFFER *b2;
-
-              /* need to switch local/remote name */
-              b1=GWEN_Buffer_new(0, 64, 0, 1);
-              b2=GWEN_Buffer_new(0, 64, 0, 1);
-
-              /* get data */
-              s=AB_Transaction_GetLocalName(t);
-              if (s && *s)
-                GWEN_Buffer_AppendString(b1, s);
-              s=AB_Transaction_GetRemoteName(t);
-              if (s && *s)
-                GWEN_Buffer_AppendString(b2, s);
-
-              /* set reverse */
-              if (GWEN_Buffer_GetUsedBytes(b1))
-                AB_Transaction_SetRemoteName(t, GWEN_Buffer_GetStart(b1));
-
-              if (GWEN_Buffer_GetUsedBytes(b2))
-                AB_Transaction_SetLocalName(t, GWEN_Buffer_GetStart(b2));
-
-              /* cleanup */
-              GWEN_Buffer_free(b2);
-              GWEN_Buffer_free(b1);
-            }
-          }
-        }
-
-        /* set transaction type if none set */
-        if (AB_Transaction_GetType(t)<=AB_Transaction_TypeNone)
-          AB_Transaction_SetType(t, AB_Transaction_TypeStatement);
-
-        /* add transaction */
-        DBG_DEBUG(AQBANKING_LOGDOMAIN, "Adding transaction");
-        AB_ImExporterContext_AddTransaction(ctx, t);
-      }
-      else {
-        DBG_ERROR(AQBANKING_LOGDOMAIN, "Empty group (i.e. empty line in imported file)");
-      }
-    }
-    else {
-      int rv;
-
-      DBG_INFO(AQBANKING_LOGDOMAIN, "Not a transaction, checking subgroups");
-      /* not a transaction, check subgroups */
-      rv=AH_ImExporterCSV__ImportFromGroup(ctx, dbT, dbParams);
-      if (rv) {
-        DBG_INFO(AQBANKING_LOGDOMAIN, "here");
-        GWEN_Gui_ProgressEnd(progressId);
-        return rv;
-      }
-    }
-
-    if (GWEN_Gui_ProgressAdvance(progressId, GWEN_GUI_PROGRESS_ONE)==
-        GWEN_ERROR_USER_ABORTED) {
-      GWEN_Gui_ProgressLog(progressId, GWEN_LoggerLevel_Error,
-                           I18N("Aborted by user"));
-      GWEN_Gui_ProgressEnd(progressId);
-      return GWEN_ERROR_USER_ABORTED;
-    }
-
-    dbT=GWEN_DB_GetNextGroup(dbT);
-  } // while
-
-  GWEN_Gui_ProgressEnd(progressId);
+  /* still undecided? */
+  if (!defaultIsPositive)
+    return 1;
 
   return 0;
 }
 
 
 
-int AH_ImExporterCSV_CheckFile(AB_IMEXPORTER *ie, const char *fname)
+void _switchLocalRemoteAccordingToSign(AB_TRANSACTION *t, int switchOnNegative)
+{
+  const AB_VALUE *pv;
+  
+  /* value must be negated, because default is negative */
+  pv=AB_Transaction_GetValue(t);
+  if (pv) {
+    if (!(AB_Value_IsNegative(pv) ^ (switchOnNegative!=0))) {
+      const char *s;
+      GWEN_BUFFER *b1;
+      GWEN_BUFFER *b2;
+  
+      /* need to switch local/remote name */
+      b1=GWEN_Buffer_new(0, 64, 0, 1);
+      b2=GWEN_Buffer_new(0, 64, 0, 1);
+  
+      /* get data */
+      s=AB_Transaction_GetLocalName(t);
+      if (s && *s)
+	GWEN_Buffer_AppendString(b1, s);
+      s=AB_Transaction_GetRemoteName(t);
+      if (s && *s)
+	GWEN_Buffer_AppendString(b2, s);
+  
+      /* set reverse */
+      if (GWEN_Buffer_GetUsedBytes(b1))
+	AB_Transaction_SetRemoteName(t, GWEN_Buffer_GetStart(b1));
+  
+      if (GWEN_Buffer_GetUsedBytes(b2))
+	AB_Transaction_SetLocalName(t, GWEN_Buffer_GetStart(b2));
+  
+      /* cleanup */
+      GWEN_Buffer_free(b2);
+      GWEN_Buffer_free(b1);
+    }
+  }
+}
+
+
+
+int _groupNameMatches(const char *groupName, GWEN_DB_NODE *dbParams)
+{
+  int i;
+  const char *p;
+  
+  for (i=0; ; i++) {
+    p=GWEN_DB_GetCharValue(dbParams, "groupNames", i, NULL);
+    if (!p)
+      break;
+    if (strcasecmp(groupName, p)==0)
+      return 1;
+  } /* for */
+  
+  if (i==0) {
+    /* no names given, check default */
+    if ((strcasecmp(groupName, "transaction")==0) ||
+	(strcasecmp(groupName, "debitnote")==0) ||
+	(strcasecmp(groupName, "line")==0))
+      return 1;
+  }
+
+  return 0;
+}
+
+
+
+
+int _checkCsv(AB_IMEXPORTER *ie, const char *fname)
 {
   AH_IMEXPORTER_CSV *ieh;
   GWEN_DBIO_CHECKFILE_RESULT rv;
@@ -526,10 +643,10 @@ int AH_ImExporterCSV_CheckFile(AB_IMEXPORTER *ie, const char *fname)
 
 
 
-int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
-                            AB_IMEXPORTER_CONTEXT *ctx,
-                            GWEN_SYNCIO *sio,
-                            GWEN_DB_NODE *params)
+int _exportCsv(AB_IMEXPORTER *ie,
+               AB_IMEXPORTER_CONTEXT *ctx,
+               GWEN_SYNCIO *sio,
+               GWEN_DB_NODE *params)
 {
   AH_IMEXPORTER_CSV *ieh;
   AB_IMEXPORTER_ACCOUNTINFO *ai;
@@ -782,10 +899,10 @@ int AH_ImExporterCSV_Export(AB_IMEXPORTER *ie,
 
 
 
-int AH_ImExporterCSV_GetEditProfileDialog(AB_IMEXPORTER *ie,
-                                          GWEN_DB_NODE *dbProfile,
-                                          const char *testFileName,
-                                          GWEN_DIALOG **pDlg)
+int _getEditProfileDialog(AB_IMEXPORTER *ie,
+                          GWEN_DB_NODE *dbProfile,
+                          const char *testFileName,
+                          GWEN_DIALOG **pDlg)
 {
   GWEN_DIALOG *dlg;
 
