@@ -1,6 +1,6 @@
 /***************************************************************************
  begin       : Tue May 03 2005
- copyright   : (C) 2005 by Martin Preuss
+ copyright   : (C) 2018 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -13,14 +13,14 @@
  * This tutorial simply creates an instance of AqBanking, initializes and  *
  * deinitializes it.                                                       *
  *                                                                         *
- * You must either choose a frontend to be used with AqBanking or create   *
- * one yourself by implementing the user interface callbacks of AqBanking. *
+ * You must either choose a GUI implementation to be used with AqBanking   *
+ * or create one yourself by implementing the user interface callbacks of  *
+ * LibGwenhywfar.                                                          *
  *                                                                         *
- * However, for simplicity reasons we use the console frontend CBanking    *
+ * However, for simplicity reasons we use the console GUI implementation   *
  * which implements these callbacks for you.                               *
  *                                                                         *
- * There are other frontends, e.g. G2Banking for GTK2/Gnome, QBanking for  *
- * QT3 or KDE3 or KBanking for KDE3.                                       *
+ * There are other GUI implementations, e.g. for GTK2, QT3, QT4 and FOX16. *
  ***************************************************************************/
 
 
@@ -33,7 +33,8 @@
 
 
 
-int main(int argc, char **argv) {
+int main(int argc, char **argv)
+{
   AB_BANKING *ab;
   int rv;
   GWEN_GUI *gui;
@@ -63,8 +64,10 @@ int main(int argc, char **argv) {
     return 2;
   }
 
+
   fprintf(stderr, "AqBanking successfully initialized.\n");
 
+
   /* You must always call this function before exiting, because only then
    * AqBanking's settings are written.
    * After this function has been called no other function except
